# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import role_create_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.role_list_response import RoleListResponse
from ..types.role_create_response import RoleCreateResponse

__all__ = ["RolesResource", "AsyncRolesResource"]


class RolesResource(SyncAPIResource):
    """Role management operations"""

    @cached_property
    def with_raw_response(self) -> RolesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fragment-dev/fragment-py#accessing-raw-response-data-eg-headers
        """
        return RolesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RolesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fragment-dev/fragment-py#with_streaming_response
        """
        return RolesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        role: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoleCreateResponse:
        """
        Creates a new role

        Args:
          role: Name of the role

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/roles",
            body=maybe_transform({"role": role}, role_create_params.RoleCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoleCreateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoleListResponse:
        """Lists all roles for the workspace"""
        return self._get(
            "/roles",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoleListResponse,
        )


class AsyncRolesResource(AsyncAPIResource):
    """Role management operations"""

    @cached_property
    def with_raw_response(self) -> AsyncRolesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fragment-dev/fragment-py#accessing-raw-response-data-eg-headers
        """
        return AsyncRolesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRolesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fragment-dev/fragment-py#with_streaming_response
        """
        return AsyncRolesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        role: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoleCreateResponse:
        """
        Creates a new role

        Args:
          role: Name of the role

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/roles",
            body=await async_maybe_transform({"role": role}, role_create_params.RoleCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoleCreateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoleListResponse:
        """Lists all roles for the workspace"""
        return await self._get(
            "/roles",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoleListResponse,
        )


class RolesResourceWithRawResponse:
    def __init__(self, roles: RolesResource) -> None:
        self._roles = roles

        self.create = to_raw_response_wrapper(
            roles.create,
        )
        self.list = to_raw_response_wrapper(
            roles.list,
        )


class AsyncRolesResourceWithRawResponse:
    def __init__(self, roles: AsyncRolesResource) -> None:
        self._roles = roles

        self.create = async_to_raw_response_wrapper(
            roles.create,
        )
        self.list = async_to_raw_response_wrapper(
            roles.list,
        )


class RolesResourceWithStreamingResponse:
    def __init__(self, roles: RolesResource) -> None:
        self._roles = roles

        self.create = to_streamed_response_wrapper(
            roles.create,
        )
        self.list = to_streamed_response_wrapper(
            roles.list,
        )


class AsyncRolesResourceWithStreamingResponse:
    def __init__(self, roles: AsyncRolesResource) -> None:
        self._roles = roles

        self.create = async_to_streamed_response_wrapper(
            roles.create,
        )
        self.list = async_to_streamed_response_wrapper(
            roles.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "TransactionCreateAllocationsParams",
    "AllocationUpdate",
    "AllocationUpdateAddAllocationOperation",
    "AllocationUpdateAddAllocationOperationUser",
    "AllocationUpdateAddAllocationOperationUserID",
    "AllocationUpdateAddAllocationOperationUserExternalID",
    "AllocationUpdateDeleteAllocationOperation",
]


class TransactionCreateAllocationsParams(TypedDict, total=False):
    allocation_updates: Required[Iterable[AllocationUpdate]]
    """Allocation operations to apply"""

    version: Required[int]
    """Current transaction version for optimistic concurrency control"""


class AllocationUpdateAddAllocationOperationUserID(TypedDict, total=False):
    id: Required[str]
    """FRAGMENT generated ID of the user"""


class AllocationUpdateAddAllocationOperationUserExternalID(TypedDict, total=False):
    external_id: Required[str]
    """External ID of the user"""


AllocationUpdateAddAllocationOperationUser: TypeAlias = Union[
    AllocationUpdateAddAllocationOperationUserID, AllocationUpdateAddAllocationOperationUserExternalID
]


class AllocationUpdateAddAllocationOperation(TypedDict, total=False):
    amount: Required[str]
    """Amount to allocate in smallest currency unit as stringified bigint."""

    invoice_id: Required[str]
    """The invoice to allocate against."""

    op: Required[Literal["add"]]
    """Add a new allocation"""

    type: Required[Literal["invoice_payin", "invoice_payout"]]
    """The type of allocation."""

    user: Required[AllocationUpdateAddAllocationOperationUser]


class AllocationUpdateDeleteAllocationOperation(TypedDict, total=False):
    id: Required[str]
    """The ID of the allocation to remove."""

    op: Required[Literal["delete"]]
    """Delete an allocation"""


AllocationUpdate: TypeAlias = Union[AllocationUpdateAddAllocationOperation, AllocationUpdateDeleteAllocationOperation]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "TransactionCreateParams",
    "Account",
    "Allocation",
    "AllocationUser",
    "AllocationUserID",
    "AllocationUserExternalID",
    "Tag",
]


class TransactionCreateParams(TypedDict, total=False):
    account: Required[Account]
    """Account reference. Provide id, external_id, or both."""

    allocations: Required[Iterable[Allocation]]
    """Allocation entries for this transaction. Empty indicates unreconciled funds."""

    amount: Required[str]
    """
    Amount in smallest currency unit as stringified bigint (can be positive or
    negative).
    """

    currency: Required[
        Literal[
            "ADA",
            "BTC",
            "DAI",
            "ETH",
            "SOL",
            "USDC",
            "USDT",
            "USDG",
            "EURC",
            "CADC",
            "CADT",
            "XLM",
            "UNI",
            "BCH",
            "LTC",
            "AAVE",
            "LINK",
            "MATIC",
            "PTS",
            "AED",
            "AFN",
            "ALL",
            "AMD",
            "ANG",
            "AOA",
            "ARS",
            "AUD",
            "AWG",
            "AZN",
            "BAM",
            "BBD",
            "BDT",
            "BGN",
            "BHD",
            "BIF",
            "BMD",
            "BND",
            "BOB",
            "BRL",
            "BSD",
            "BTN",
            "BWP",
            "BYR",
            "BZD",
            "CAD",
            "CDF",
            "CHF",
            "CLP",
            "CNY",
            "COP",
            "CRC",
            "CUC",
            "CUP",
            "CVE",
            "CZK",
            "DJF",
            "DKK",
            "DOP",
            "DZD",
            "EGP",
            "ERN",
            "ETB",
            "EUR",
            "FJD",
            "FKP",
            "GBP",
            "GEL",
            "GGP",
            "GHS",
            "GIP",
            "GMD",
            "GNF",
            "GTQ",
            "GYD",
            "HKD",
            "HNL",
            "HRK",
            "HTG",
            "HUF",
            "IDR",
            "ILS",
            "IMP",
            "INR",
            "IQD",
            "IRR",
            "ISK",
            "JMD",
            "JOD",
            "JPY",
            "KES",
            "KGS",
            "KHR",
            "KMF",
            "KPW",
            "KRW",
            "KWD",
            "KYD",
            "KZT",
            "LAK",
            "LBP",
            "LKR",
            "LRD",
            "LSL",
            "LYD",
            "MAD",
            "MDL",
            "MGA",
            "MKD",
            "MMK",
            "MNT",
            "MOP",
            "MUR",
            "MVR",
            "MWK",
            "MXN",
            "MYR",
            "MZN",
            "NAD",
            "NGN",
            "NIO",
            "NOK",
            "NPR",
            "NZD",
            "OMR",
            "PAB",
            "PEN",
            "PGK",
            "PHP",
            "PKR",
            "PLN",
            "PYG",
            "QAR",
            "RON",
            "RSD",
            "RUB",
            "RWF",
            "SAR",
            "SBD",
            "SCR",
            "SDG",
            "SEK",
            "SGD",
            "SHP",
            "SLL",
            "SOS",
            "SPL",
            "SRD",
            "SVC",
            "SYP",
            "STN",
            "SZL",
            "THB",
            "TJS",
            "TMT",
            "TND",
            "TOP",
            "TRY",
            "TTD",
            "TVD",
            "TWD",
            "TZS",
            "UAH",
            "UGX",
            "USD",
            "UYU",
            "UZS",
            "VEF",
            "VND",
            "VUV",
            "WST",
            "XAF",
            "XCD",
            "XOF",
            "XPF",
            "YER",
            "ZAR",
            "ZMW",
            "LOGICAL",
            "CUSTOM",
        ]
    ]
    """Currency code (ISO 4217 or crypto)"""

    external_id: Required[str]
    """External transaction ID used for idempotent sync."""

    posted: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """Posted timestamp in ISO 8601 format."""

    tags: Iterable[Tag]
    """Optional metadata tags for this transaction"""


class Account(TypedDict, total=False):
    """Account reference. Provide id, external_id, or both."""

    id: str
    """User-facing encoded account ID."""

    external_id: str
    """External account reference ID."""


class AllocationUserID(TypedDict, total=False):
    id: Required[str]
    """FRAGMENT generated ID of the user"""


class AllocationUserExternalID(TypedDict, total=False):
    external_id: Required[str]
    """External ID of the user"""


AllocationUser: TypeAlias = Union[AllocationUserID, AllocationUserExternalID]


class Allocation(TypedDict, total=False):
    """Transaction allocation against an invoice."""

    amount: Required[str]
    """Amount to allocate in smallest currency unit as stringified bigint."""

    invoice_id: Required[str]
    """The invoice to allocate against."""

    type: Required[Literal["invoice_payin", "invoice_payout"]]
    """The type of allocation."""

    user: Required[AllocationUser]


class Tag(TypedDict, total=False):
    """A key-value tag pair for metadata"""

    key: Required[str]
    """Tag key.

    Must be a valid safe string (no special characters like #, /, :). Max 50
    characters.
    """

    value: Required[str]
    """Tag value.

    Must be a valid safe string (no special characters like #, /, :). Max 200
    characters.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from fragment import Fragment, AsyncFragment
from tests.utils import assert_matches_type
from fragment.types import (
    TransactionListResponse,
    TransactionCreateResponse,
    TransactionSearchResponse,
    TransactionRetrieveResponse,
    TransactionListHistoryResponse,
    TransactionCreateAllocationsResponse,
    TransactionSearchAllocationsResponse,
)
from fragment._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTransactions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Fragment) -> None:
        transaction = client.transactions.create(
            account={},
            allocations=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            amount="-1000",
            currency="USD",
            external_id="bank_txn_123",
            posted=parse_datetime("2026-02-12T00:00:00.000Z"),
        )
        assert_matches_type(TransactionCreateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Fragment) -> None:
        transaction = client.transactions.create(
            account={
                "id": "ext_account_YWJjMTIz",
                "external_id": "acct_external_123",
            },
            allocations=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            amount="-1000",
            currency="USD",
            external_id="bank_txn_123",
            posted=parse_datetime("2026-02-12T00:00:00.000Z"),
            tags=[
                {
                    "key": "region",
                    "value": "us-east",
                }
            ],
        )
        assert_matches_type(TransactionCreateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Fragment) -> None:
        response = client.transactions.with_raw_response.create(
            account={},
            allocations=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            amount="-1000",
            currency="USD",
            external_id="bank_txn_123",
            posted=parse_datetime("2026-02-12T00:00:00.000Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionCreateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Fragment) -> None:
        with client.transactions.with_streaming_response.create(
            account={},
            allocations=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            amount="-1000",
            currency="USD",
            external_id="bank_txn_123",
            posted=parse_datetime("2026-02-12T00:00:00.000Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionCreateResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Fragment) -> None:
        transaction = client.transactions.retrieve(
            "txn_abc123",
        )
        assert_matches_type(TransactionRetrieveResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Fragment) -> None:
        response = client.transactions.with_raw_response.retrieve(
            "txn_abc123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionRetrieveResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Fragment) -> None:
        with client.transactions.with_streaming_response.retrieve(
            "txn_abc123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionRetrieveResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Fragment) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_ref` but received ''"):
            client.transactions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Fragment) -> None:
        transaction = client.transactions.list()
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Fragment) -> None:
        transaction = client.transactions.list(
            account="ext_account_YWJjMTIz",
            reconciliation_status="reconciled",
        )
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Fragment) -> None:
        response = client.transactions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Fragment) -> None:
        with client.transactions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionListResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_allocations(self, client: Fragment) -> None:
        transaction = client.transactions.create_allocations(
            transaction_ref="txn_abc123",
            allocation_updates=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "op": "add",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            version=0,
        )
        assert_matches_type(TransactionCreateAllocationsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_allocations(self, client: Fragment) -> None:
        response = client.transactions.with_raw_response.create_allocations(
            transaction_ref="txn_abc123",
            allocation_updates=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "op": "add",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            version=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionCreateAllocationsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_allocations(self, client: Fragment) -> None:
        with client.transactions.with_streaming_response.create_allocations(
            transaction_ref="txn_abc123",
            allocation_updates=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "op": "add",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            version=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionCreateAllocationsResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create_allocations(self, client: Fragment) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_ref` but received ''"):
            client.transactions.with_raw_response.create_allocations(
                transaction_ref="",
                allocation_updates=[
                    {
                        "amount": "1000",
                        "invoice_id": "inv_abc123",
                        "op": "add",
                        "type": "invoice_payin",
                        "user": {"id": "user_abc123"},
                    }
                ],
                version=0,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_history(self, client: Fragment) -> None:
        transaction = client.transactions.list_history(
            "txn_abc123",
        )
        assert_matches_type(TransactionListHistoryResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_history(self, client: Fragment) -> None:
        response = client.transactions.with_raw_response.list_history(
            "txn_abc123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionListHistoryResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_history(self, client: Fragment) -> None:
        with client.transactions.with_streaming_response.list_history(
            "txn_abc123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionListHistoryResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list_history(self, client: Fragment) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_ref` but received ''"):
            client.transactions.with_raw_response.list_history(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_search(self, client: Fragment) -> None:
        transaction = client.transactions.search(
            filter={"account": {"any": [{}]}},
        )
        assert_matches_type(TransactionSearchResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_search(self, client: Fragment) -> None:
        response = client.transactions.with_raw_response.search(
            filter={"account": {"any": [{}]}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionSearchResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_search(self, client: Fragment) -> None:
        with client.transactions.with_streaming_response.search(
            filter={"account": {"any": [{}]}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionSearchResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_search_allocations(self, client: Fragment) -> None:
        transaction = client.transactions.search_allocations(
            filter={"invoice_id": {"any": ["inv_abc123"]}},
        )
        assert_matches_type(TransactionSearchAllocationsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_search_allocations(self, client: Fragment) -> None:
        response = client.transactions.with_raw_response.search_allocations(
            filter={"invoice_id": {"any": ["inv_abc123"]}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(TransactionSearchAllocationsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_search_allocations(self, client: Fragment) -> None:
        with client.transactions.with_streaming_response.search_allocations(
            filter={"invoice_id": {"any": ["inv_abc123"]}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(TransactionSearchAllocationsResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTransactions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncFragment) -> None:
        transaction = await async_client.transactions.create(
            account={},
            allocations=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            amount="-1000",
            currency="USD",
            external_id="bank_txn_123",
            posted=parse_datetime("2026-02-12T00:00:00.000Z"),
        )
        assert_matches_type(TransactionCreateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncFragment) -> None:
        transaction = await async_client.transactions.create(
            account={
                "id": "ext_account_YWJjMTIz",
                "external_id": "acct_external_123",
            },
            allocations=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            amount="-1000",
            currency="USD",
            external_id="bank_txn_123",
            posted=parse_datetime("2026-02-12T00:00:00.000Z"),
            tags=[
                {
                    "key": "region",
                    "value": "us-east",
                }
            ],
        )
        assert_matches_type(TransactionCreateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncFragment) -> None:
        response = await async_client.transactions.with_raw_response.create(
            account={},
            allocations=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            amount="-1000",
            currency="USD",
            external_id="bank_txn_123",
            posted=parse_datetime("2026-02-12T00:00:00.000Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionCreateResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncFragment) -> None:
        async with async_client.transactions.with_streaming_response.create(
            account={},
            allocations=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            amount="-1000",
            currency="USD",
            external_id="bank_txn_123",
            posted=parse_datetime("2026-02-12T00:00:00.000Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionCreateResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncFragment) -> None:
        transaction = await async_client.transactions.retrieve(
            "txn_abc123",
        )
        assert_matches_type(TransactionRetrieveResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncFragment) -> None:
        response = await async_client.transactions.with_raw_response.retrieve(
            "txn_abc123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionRetrieveResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncFragment) -> None:
        async with async_client.transactions.with_streaming_response.retrieve(
            "txn_abc123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionRetrieveResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncFragment) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_ref` but received ''"):
            await async_client.transactions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncFragment) -> None:
        transaction = await async_client.transactions.list()
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncFragment) -> None:
        transaction = await async_client.transactions.list(
            account="ext_account_YWJjMTIz",
            reconciliation_status="reconciled",
        )
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncFragment) -> None:
        response = await async_client.transactions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionListResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncFragment) -> None:
        async with async_client.transactions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionListResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_allocations(self, async_client: AsyncFragment) -> None:
        transaction = await async_client.transactions.create_allocations(
            transaction_ref="txn_abc123",
            allocation_updates=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "op": "add",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            version=0,
        )
        assert_matches_type(TransactionCreateAllocationsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_allocations(self, async_client: AsyncFragment) -> None:
        response = await async_client.transactions.with_raw_response.create_allocations(
            transaction_ref="txn_abc123",
            allocation_updates=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "op": "add",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            version=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionCreateAllocationsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_allocations(self, async_client: AsyncFragment) -> None:
        async with async_client.transactions.with_streaming_response.create_allocations(
            transaction_ref="txn_abc123",
            allocation_updates=[
                {
                    "amount": "1000",
                    "invoice_id": "inv_abc123",
                    "op": "add",
                    "type": "invoice_payin",
                    "user": {"id": "user_abc123"},
                }
            ],
            version=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionCreateAllocationsResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create_allocations(self, async_client: AsyncFragment) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_ref` but received ''"):
            await async_client.transactions.with_raw_response.create_allocations(
                transaction_ref="",
                allocation_updates=[
                    {
                        "amount": "1000",
                        "invoice_id": "inv_abc123",
                        "op": "add",
                        "type": "invoice_payin",
                        "user": {"id": "user_abc123"},
                    }
                ],
                version=0,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_history(self, async_client: AsyncFragment) -> None:
        transaction = await async_client.transactions.list_history(
            "txn_abc123",
        )
        assert_matches_type(TransactionListHistoryResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_history(self, async_client: AsyncFragment) -> None:
        response = await async_client.transactions.with_raw_response.list_history(
            "txn_abc123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionListHistoryResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_history(self, async_client: AsyncFragment) -> None:
        async with async_client.transactions.with_streaming_response.list_history(
            "txn_abc123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionListHistoryResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list_history(self, async_client: AsyncFragment) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `transaction_ref` but received ''"):
            await async_client.transactions.with_raw_response.list_history(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_search(self, async_client: AsyncFragment) -> None:
        transaction = await async_client.transactions.search(
            filter={"account": {"any": [{}]}},
        )
        assert_matches_type(TransactionSearchResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_search(self, async_client: AsyncFragment) -> None:
        response = await async_client.transactions.with_raw_response.search(
            filter={"account": {"any": [{}]}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionSearchResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncFragment) -> None:
        async with async_client.transactions.with_streaming_response.search(
            filter={"account": {"any": [{}]}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionSearchResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_search_allocations(self, async_client: AsyncFragment) -> None:
        transaction = await async_client.transactions.search_allocations(
            filter={"invoice_id": {"any": ["inv_abc123"]}},
        )
        assert_matches_type(TransactionSearchAllocationsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_search_allocations(self, async_client: AsyncFragment) -> None:
        response = await async_client.transactions.with_raw_response.search_allocations(
            filter={"invoice_id": {"any": ["inv_abc123"]}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(TransactionSearchAllocationsResponse, transaction, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_search_allocations(self, async_client: AsyncFragment) -> None:
        async with async_client.transactions.with_streaming_response.search_allocations(
            filter={"invoice_id": {"any": ["inv_abc123"]}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(TransactionSearchAllocationsResponse, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .external_account import ExternalAccount

__all__ = ["ExternalAccountListResponse"]


class ExternalAccountListResponse(BaseModel):
    """List of external accounts"""

    data: List[ExternalAccount]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .roles import (
    RolesResource,
    AsyncRolesResource,
    RolesResourceWithRawResponse,
    AsyncRolesResourceWithRawResponse,
    RolesResourceWithStreamingResponse,
    AsyncRolesResourceWithStreamingResponse,
)
from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .invoices import (
    InvoicesResource,
    AsyncInvoicesResource,
    InvoicesResourceWithRawResponse,
    AsyncInvoicesResourceWithRawResponse,
    InvoicesResourceWithStreamingResponse,
    AsyncInvoicesResourceWithStreamingResponse,
)
from .products import (
    ProductsResource,
    AsyncProductsResource,
    ProductsResourceWithRawResponse,
    AsyncProductsResourceWithRawResponse,
    ProductsResourceWithStreamingResponse,
    AsyncProductsResourceWithStreamingResponse,
)
from .transactions import (
    TransactionsResource,
    AsyncTransactionsResource,
    TransactionsResourceWithRawResponse,
    AsyncTransactionsResourceWithRawResponse,
    TransactionsResourceWithStreamingResponse,
    AsyncTransactionsResourceWithStreamingResponse,
)
from .external_accounts import (
    ExternalAccountsResource,
    AsyncExternalAccountsResource,
    ExternalAccountsResourceWithRawResponse,
    AsyncExternalAccountsResourceWithRawResponse,
    ExternalAccountsResourceWithStreamingResponse,
    AsyncExternalAccountsResourceWithStreamingResponse,
)

__all__ = [
    "ExternalAccountsResource",
    "AsyncExternalAccountsResource",
    "ExternalAccountsResourceWithRawResponse",
    "AsyncExternalAccountsResourceWithRawResponse",
    "ExternalAccountsResourceWithStreamingResponse",
    "AsyncExternalAccountsResourceWithStreamingResponse",
    "InvoicesResource",
    "AsyncInvoicesResource",
    "InvoicesResourceWithRawResponse",
    "AsyncInvoicesResourceWithRawResponse",
    "InvoicesResourceWithStreamingResponse",
    "AsyncInvoicesResourceWithStreamingResponse",
    "ProductsResource",
    "AsyncProductsResource",
    "ProductsResourceWithRawResponse",
    "AsyncProductsResourceWithRawResponse",
    "ProductsResourceWithStreamingResponse",
    "AsyncProductsResourceWithStreamingResponse",
    "RolesResource",
    "AsyncRolesResource",
    "RolesResourceWithRawResponse",
    "AsyncRolesResourceWithRawResponse",
    "RolesResourceWithStreamingResponse",
    "AsyncRolesResourceWithStreamingResponse",
    "TransactionsResource",
    "AsyncTransactionsResource",
    "TransactionsResourceWithRawResponse",
    "AsyncTransactionsResourceWithRawResponse",
    "TransactionsResourceWithStreamingResponse",
    "AsyncTransactionsResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["TransactionSearchParams", "Filter", "FilterAccount", "FilterAccountAny"]


class TransactionSearchParams(TypedDict, total=False):
    filter: Required[Filter]
    """Filter criteria for searching transactions."""


class FilterAccountAny(TypedDict, total=False):
    """Account reference. Provide id, external_id, or both."""

    id: str
    """User-facing encoded account ID."""

    external_id: str
    """External account reference ID."""


class FilterAccount(TypedDict, total=False):
    any: Required[Iterable[FilterAccountAny]]
    """Match transactions belonging to any of these accounts (OR)."""


class Filter(TypedDict, total=False):
    """Filter criteria for searching transactions."""

    account: Required[FilterAccount]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import external_account_create_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.external_account_list_response import ExternalAccountListResponse
from ..types.external_account_create_response import ExternalAccountCreateResponse

__all__ = ["ExternalAccountsResource", "AsyncExternalAccountsResource"]


class ExternalAccountsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ExternalAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fragment-dev/fragment-py#accessing-raw-response-data-eg-headers
        """
        return ExternalAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExternalAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fragment-dev/fragment-py#with_streaming_response
        """
        return ExternalAccountsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        external_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalAccountCreateResponse:
        """
        Creates a new external account

        Args:
          external_id: External ID for the account (user-provided, unique, mutable)

          name: Human-readable name for the external account (mutable)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/external-accounts",
            body=maybe_transform(
                {
                    "external_id": external_id,
                    "name": name,
                },
                external_account_create_params.ExternalAccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalAccountCreateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalAccountListResponse:
        """Lists all external accounts for the workspace"""
        return self._get(
            "/external-accounts",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalAccountListResponse,
        )


class AsyncExternalAccountsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncExternalAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fragment-dev/fragment-py#accessing-raw-response-data-eg-headers
        """
        return AsyncExternalAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExternalAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fragment-dev/fragment-py#with_streaming_response
        """
        return AsyncExternalAccountsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        external_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalAccountCreateResponse:
        """
        Creates a new external account

        Args:
          external_id: External ID for the account (user-provided, unique, mutable)

          name: Human-readable name for the external account (mutable)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/external-accounts",
            body=await async_maybe_transform(
                {
                    "external_id": external_id,
                    "name": name,
                },
                external_account_create_params.ExternalAccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalAccountCreateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalAccountListResponse:
        """Lists all external accounts for the workspace"""
        return await self._get(
            "/external-accounts",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalAccountListResponse,
        )


class ExternalAccountsResourceWithRawResponse:
    def __init__(self, external_accounts: ExternalAccountsResource) -> None:
        self._external_accounts = external_accounts

        self.create = to_raw_response_wrapper(
            external_accounts.create,
        )
        self.list = to_raw_response_wrapper(
            external_accounts.list,
        )


class AsyncExternalAccountsResourceWithRawResponse:
    def __init__(self, external_accounts: AsyncExternalAccountsResource) -> None:
        self._external_accounts = external_accounts

        self.create = async_to_raw_response_wrapper(
            external_accounts.create,
        )
        self.list = async_to_raw_response_wrapper(
            external_accounts.list,
        )


class ExternalAccountsResourceWithStreamingResponse:
    def __init__(self, external_accounts: ExternalAccountsResource) -> None:
        self._external_accounts = external_accounts

        self.create = to_streamed_response_wrapper(
            external_accounts.create,
        )
        self.list = to_streamed_response_wrapper(
            external_accounts.list,
        )


class AsyncExternalAccountsResourceWithStreamingResponse:
    def __init__(self, external_accounts: AsyncExternalAccountsResource) -> None:
        self._external_accounts = external_accounts

        self.create = async_to_streamed_response_wrapper(
            external_accounts.create,
        )
        self.list = async_to_streamed_response_wrapper(
            external_accounts.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .user import User
from .._models import BaseModel

__all__ = ["UserCreateResponse"]


class UserCreateResponse(BaseModel):
    data: User
    """User object"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel
from .external_account import ExternalAccount

__all__ = ["ExternalAccountCreateResponse"]


class ExternalAccountCreateResponse(BaseModel):
    data: ExternalAccount
    """External account object"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .product import Product
from .._models import BaseModel

__all__ = ["ProductRetrieveResponse"]


class ProductRetrieveResponse(BaseModel):
    data: Product
    """Product object"""
